<?php
if (!defined('BASEPATH')) exit('No direct script access allowed');

class Kategori_model extends Model {	
	function Kategori_model(){
		parent::Model();
	}
	
	function validation(){
		$name = $this->input->post('catname');
		if( trim($name) == '' ){
			$this->message['text'] = 'Nama kategori tidak boleh kosong.';
			$this->message['class'] = 'error';
			return false;
		}
				
		return true;
	}
	
	function load_list($parentid, $ordersegment=5){
		$this->load->library('pagination');
		
		$this->db->select('COUNT(catid)');
		$r = $this->db->getwhere('kategori_iklan', "parentid=$parentid", 1);
		
		$config['base_url'] = $this->uri->slash_segment(1, 'leading').$this->uri->slash_segment(2, 'leading').$this->uri->slash_segment(3, 'both').$parentid.'/';
		$config['total_rows'] = $r->get_one();
		$config['per_page'] = 20;
		$config['uri_segment'] = $ordersegment;	
		$this->pagination->initialize($config);
		
		$limit = $this->pagination->get_limit();
		$offset = $this->pagination->get_offset();
		
		$this->db->select();
		$this->db->orderby('catname');
		$r = $this->db->getwhere('kategori_iklan', "parentid=".$this->db->escape($parentid), $limit, $offset);
		
		$rows =  $r->result();
		
		return $rows;
	}
	
	function load($id){
		if( empty($id) ){
			$row = new stdClass();
			$row->id = '';
		} else {
			$this->db->select();
			$r = $this->db->getwhere('kategori_iklan', "catid=".$this->db->escape($id), 1);
			$row = $r->row();
		}
		
		return $row;
	}
	
	function load_all($parentid=0){
		$this->db->select('catid, catname');
		$this->db->orderby('catname');
		$r = $this->db->getwhere('kategori_iklan', "parentid=".$this->db->escape($parentid));
		if( $r === false )
			return array();
		
		$rows = array();
		foreach( $r->result() as $row ){
			$row->submenu = $this->load_all($row->catid);
			$rows[] = $row;
		}
		
		return $rows;
	}
	
	function store(){
		$data['catid'] = $this->input->post('id');
		$data['catname'] = $this->input->post('catname');
		$data['published'] = $this->input->post('published') ? $this->input->post('published'):'N';
		$data['parentid'] = $this->input->post('parentid');
		$data['slug'] = url_title($data['catname']);
		if( $data['catid'] ){
			return $this->update($data);
		} else {
			return $this->insert($data);
		}
	}
	
	function insert($post){
		$r = $this->db->insert('kategori_iklan', $post);
		if( $r === false ){
			$this->message['text'] = 'Kategori iklan gagal disimpan';
			$this->message['class'] = 'error';
			return false;
		}
		$this->message['text'] = 'Kategori iklan berhasil disimpan';
		$this->message['class'] = 'ok';
		return true;
	}
	
	function update($post){
		$id = $post['catid'];
		unset($post['id']);
		$r = $this->db->update('kategori_iklan', $post, "catid=".$this->db->escape($id));
		if( $r === false ){
			$this->message['text'] = 'Kategori iklan gagal disimpan';
			$this->message['class'] = 'error';
			return false;
		}
		$this->message['text'] = 'Kategori iklan berhasil disimpan';
		$this->message['class'] = 'ok';
		return true;
	}
	
	function published($published){
		$ids = $this->input->post('id');
		foreach( $ids as $id ){
			$r = $this->db->update('kategori_iklan', array('published'=>"$published"), "catid=".$this->db->escape($id));
		}
		
		return true;
	}
	
	function delete(){
		$ids = $this->input->post('id');
		$ftotal = 0;
		$stotal = 0;
		foreach( $ids as $key => $id ){
			$r = $this->db->delete('kategori_iklan', "catid=".$this->db->escape($id));
			if( $r === false ){
				$ftotal++;
			} else {
				$stotal++;
			}
		}
		
		$msg = '';
		if( $stotal > 0 ){
			$msg .= 'Berhasil menghapus '.$stotal.' data<br />';
		}
		if( $ftotal > 0 ){
			$msg .= 'Gagal menghapus '.$ftotal.' data';
		}
		
		$this->message['text'] = $msg;
		$this->message['class'] = 'ok';
		
		return true;
	}
	
	function delete_sub($parentid){
		$this->db->select('catid');
		$r = $this->db->getwhere('kategori_iklan', "parentid=".$this->db->escape($parentid));
		$subs = array();
		foreach( $r->result() as $row ){
			$subs[] = $row->catid;
		}
	}
	
	function buttonlist($parentid){
		$btn = $this->button->start();
		if( current_user_permision('write') === true ){
			$btn .= $this->button->add_button("/backend/ads/kategori_add/$parentid/");
			$btn .= $this->button->separator();
		}
		if( current_user_permision('modify') === true ){
			$btn .= $this->button->published_button();
			$btn .= $this->button->separator();
			$btn .= $this->button->unpublished_button();
			$btn .= $this->button->separator();
		}
		if( current_user_permision('delete') === true ){
			$btn .= $this->button->delete_button();
		}
		$btn .= $this->button->end();
		
		return $btn;
	}
	
	function buttonadd($parentid){
		$btn = $this->button->start();
		$btn .= $this->button->save_button();
		$btn .= $this->button->separator();
		$btn .= $this->button->cancel_button("/backend/ads/kategori/$parentid/");
		$btn .= $this->button->end();
		
		return $btn;
	}
}
?>