<?php
if (!defined('BASEPATH')) exit('No direct script access allowed');

class Seed_model extends Model {
	function Seed_model(){
		parent::Model();
	}

	function validate(){
		$catid = $this->_get_catid_by_name($this->input->post('catname', true));
		$name = strip_tags($this->input->post('name', true));
		$email = $this->input->post('email', true);
		$title = $this->input->post('title', true);
		$content = $this->input->post('content', true);
		
		if( empty($catid) || empty($name) || empty($email) || empty($title) || empty($content) ){
			return false;
		}
		
		return true;
	}
	
	function store(){
		if( $this->validate() === false ){
			return false;
		}

		$post['catid'] = $this->_get_catid_by_name($this->input->post('catname', true));
		$post['name'] = $this->input->post('name', true);
		$post['email'] = $this->input->post('email', true);
		$post['title'] = $this->input->post('title', true);
		$post['content'] = $this->input->post('content', true);
		$post['website'] = $this->input->post('website', true);
		$post['typeiklan'] = 'free';
		$tags = $this->input->post('tags', true);
		
		$id = $this->_check_iklan($post);
		
		if( !$id ){
			$post['created_date'] = now();
			$post['slug'] = $this->_create_slug($this->input->post('title'));
			
			$r = $this->db->insert('iklan_baris', $post);
			
			if( !$r ){
				$this->message = 'Iklan Anda gagal disimpan.<br />Silahkan hubungi administrator untuk melaporkan hal ini.';
				return false;
			}
			
			$id = $this->db->insert_id();
		} else {
			$update['created_date'] = now();
			
			$r = $this->db->update('iklan_baris', $update, "id=$id");
			if( !$r ){
				$this->message = 'Iklan Anda gagal disimpan.<br />Silahkan hubungi administrator untuk melaporkan hal ini.';
				return false;
			}
		}
		
		$this->iklanid = $id;
		$this->_parse_tags($tags, $post['title']);
		$this->_store_tags();
		$this->_assign_tags_to_ads();
		
		return true;
	}
	
	function _check_iklan($post){
		$check['content'] = $post['content'];
		$check['typeiklan'] = 'free';
		$check['title'] = $post['title'];
		$check['website'] = $post['website'];
		
		$this->db->select('id');
		$r = $this->db->getwhere('iklan_baris', $check, 1);
		$id = $r->get_one();
	
		return $id;
	}

	function _create_slug($str, $num=1, $ori=''){
		$slug = url_title($str);
		
		if( $ori == '' ){
			$ori = $str;
		}
		
		$this->db->select('id');
		$r = $this->db->getwhere('iklan_baris', "slug=".$this->db->escape($slug));
		$id = $r->get_one();
		if( !$id ){
			return $slug;
		}
		
		$slug = $ori.'-'.$num;
		$num++;
		
		return $this->_create_slug($slug, $num, $ori);
	}

	function _parse_tags($tags, $title){
		$this->tags = array();
		$tags = strip_tags(strtolower($tags));
		if( trim(strip_tags($tags)) == '' ){
			$title = strip_tags(strtolower($title));
			$tags = preg_replace("#\s+#", ',', $title);
		}
		$tags = preg_replace("#[^A-Za-z0-9,\s]#", '', $tags);
		$tags = explode(',', strtolower($tags));
		
		$this->_parse_banned_word();
		
		$i=1;
		foreach($tags as $tag){
			$tag = trim(strtolower($tag));
			if( $tag != '' && !in_array($tag, $this->tags) && strlen($tag) >= 3 && !in_array($tag, $this->bannedwords) ){
				$this->tags[] = $tag;
				$i++;
			}
			
			if( $i > 5 )
				break;
		}
		
		array_unique($this->tags);
	}

	function _parse_banned_word(){
		$banned = $this->config->item('ads_bannedword');
		
		$this->bannedwords = explode("\n", str_replace("\r\n", "\n", $banned));
		array_unique($this->bannedwords);
	}

	function _store_tags(){
		$this->tags_id = array();
		foreach( $this->tags as $tag ){
			$tagslug = url_title($tag);
			$this->db->select('tagid');
			$r = $this->db->getwhere('tags', "slug=".$this->db->escape($tagslug), 1);
			$id = $r->get_one();
			
			if( !$id ){
				$data['tag'] = $tag;
				$data['counter'] = 1;
				$data['slug'] = $tagslug;
				$r = $this->db->insert('tags', $data);
				$id = $this->db->insert_id();
			} else {
				$this->db->select('1');
				$r = $this->db->getwhere('iklan2tag', "tagid=$id AND iklanid=".$this->iklanid, 1);
				$e = $r->get_one();
				if( !$e ){
					$this->db->select('counter');
					$r = $this->db->getwhere('tags', "tagid=$id", 1);
					$counter = $r->get_one();
					$update['counter'] = $counter + 1;
					$r = $this->db->update('tags', $update, "tagid=$id");
				}
			}
			$this->tags_id[] = $id;
		}
	}
	
	function _assign_tags_to_ads(){
		foreach( (array) $this->tags_id as $tag ){
			$this->db->select('1');
			$r = $this->db->getwhere('iklan2tag', "tagid=$tag AND iklanid=".$this->iklanid, 1);
			$e = $r->get_one();
			if( !$e ){
				$data['tagid'] = $tag;
				$data['iklanid'] = $this->iklanid;
				$this->db->insert('iklan2tag', $data);
			}
		}
	}
	
	function _get_catid_by_name($name){
		$this->db->select('catid');
		$r = $this->db->getwhere('kategori_iklan', "catname LIKE '%".$name."%'");
		$e = $r->get_one();
		if( !$e ){
			$r = $this->db->getwhere('kategori_iklan', "catname LIKE '%lain%'");
			$e = $r->get_one();
		}

		return $e;
	}


}
?>