<?php
define('CAPTCHA_SESSION_ID', 'php_captcha');
define('CAPTCHA_WIDTH', 150); // max 500
define('CAPTCHA_HEIGHT', 50); // max 200
define('CAPTCHA_NUM_CHARS', 5);
define('CAPTCHA_NUM_LINES', 30);
define('CAPTCHA_CHAR_SHADOW', false);
define('CAPTCHA_OWNER_TEXT', '');
define('CAPTCHA_CHAR_SET', '2,3,4,5,6,7,8,9,a,b,c,d,e,f,g,h,i,j,k,m,n,p,q,r,s,t,u,v,w,x,y,z,A,B,C,D,E,F,G,H,I,J,K,M,N,P,Q,R,S,V,W,X,Y,Z'); // defaults to A-Z
define('CAPTCHA_CASE_INSENSITIVE', true);
define('CAPTCHA_BACKGROUND_IMAGES', '');
define('CAPTCHA_MIN_FONT_SIZE', 23);
define('CAPTCHA_MAX_FONT_SIZE', 25);
define('CAPTCHA_USE_COLOUR', true);
define('CAPTCHA_FILE_TYPE', 'jpeg');
define('CAPTCHA_FLITE_PATH', '/usr/bin/flite');
define('CAPTCHA_AUDIO_PATH', '/tmp/'); // must be writeable by PHP process
define('CAPTCHA_DIR', dirname(__FILE__) );

class Iwmcaptcha {
  	var $oImage;
  	var $aFonts;
  	var $iWidth;
  	var $iHeight;
  	var $iNumChars;
  	var $iNumLines;
  	var $iSpacing;
  	var $bCharShadow;
  	var $sOwnerText;
  	var $aCharSet;
  	var $bCaseInsensitive;
  	var $vBackgroundImages;
  	var $iMinFontSize;
  	var $iMaxFontSize;
  	var $bUseColour;
  	var $sFileType;
  	var $sCode = '';
  
  function Iwmcaptcha($params=array()) {
	 // get parameters
	 $this->obj =& get_instance();
	 $this->initialize($params);
  }
  
  function initialize($params){
  	 $this->SetFonts();
	 $this->SetNumChars(CAPTCHA_NUM_CHARS);
	 
  	 if( is_array($params) ){
		foreach($params as $key => $val ){
			$method = 'set_'.$key;
			if (method_exists($this, $method))
			{
				$this->$method($val);
			}
			else
			{
				$this->$key = $config[$key];
			}
		}
	 }
	 
	 $this->SetNumLines(CAPTCHA_NUM_LINES);
	 $this->DisplayShadow(CAPTCHA_CHAR_SHADOW);
	 $this->SetOwnerText(CAPTCHA_OWNER_TEXT);
	 $this->SetCharSet(CAPTCHA_CHAR_SET);
	 $this->CaseInsensitive(CAPTCHA_CASE_INSENSITIVE);
	 $this->SetBackgroundImages(CAPTCHA_BACKGROUND_IMAGES);
	 $this->SetMinFontSize(CAPTCHA_MIN_FONT_SIZE);
	 $this->SetMaxFontSize(CAPTCHA_MAX_FONT_SIZE);
	 $this->UseColour(CAPTCHA_USE_COLOUR);
	 $this->SetFileType(CAPTCHA_FILE_TYPE); 
  }
  
  function set_sessvar($sessvar){
  	 if( trim($sessvar) == '' )
	 	$sessvar = CAPTCHA_SESSION_ID;
	
	 $this->sessvar = $sessvar;
  }
  
  function SetFonts(){
  	 $this->fontPath = BASEPATH.'fonts/';
	 $fonts = array();
	 $d = dir($this->fontPath);
	 while ( false !== ($entry = $d->read()) ) {
		if( $entry == '.' || $entry == '..' )
			continue;
		
		if( is_dir($this->fontPath.$entry) )
			continue;
		
		if( ereg('.ttf', strtolower($entry)) )
			$fonts[] = $entry;
	 }
	 $d->close();
	 
  	 $this->aFonts = $fonts;
  }
  
  function CalculateSpacing() {
	 $this->iSpacing = (int)($this->iWidth / $this->iNumChars);
  }
  
  function set_width($iWidth) {
  	 if( empty($iWidth) )
	 	$iWidth = CAPTCHA_WIDTH;
	
	 $this->iWidth = $iWidth;
	 if ($this->iWidth > 500) $this->iWidth = 500; // to prevent perfomance impact
	 	$this->CalculateSpacing();
  }
  
  function set_height($iHeight) {
  	 if( empty($iHeight) )
	 	$iHeight = CAPTCHA_HEIGHT;
		
	 $this->iHeight = $iHeight;
	 if ($this->iHeight > 200) $this->iHeight = 200; // to prevent performance impact
  }
  
  function SetNumChars($iNumChars) {
	 $this->iNumChars = $iNumChars;
	 $this->CalculateSpacing();
  }
  
  function SetNumLines($iNumLines) {
	 $this->iNumLines = $iNumLines;
  }
  
  function DisplayShadow($bCharShadow) {
	 $this->bCharShadow = $bCharShadow;
  }
  
  function SetOwnerText($sOwnerText) {
	 $this->sOwnerText = $sOwnerText;
  }
  
  function SetCharSet($vCharSet) {
	 // check for input type
	 if (is_array($vCharSet)) {
		$this->aCharSet = $vCharSet;
	 } else {
		if ($vCharSet != '') {
		   // split items on commas
		   $aCharSet = explode(',', $vCharSet);
		
		   // initialise array
		   $this->aCharSet = array();
		
		   // loop through items 
		   foreach ($aCharSet as $sCurrentItem) {
			  // a range should have 3 characters, otherwise is normal character
			  if (strlen($sCurrentItem) == 3) {
				 // split on range character
				 $aRange = explode('-', $sCurrentItem);
			  
				 // check for valid range
				 if (count($aRange) == 2 && $aRange[0] < $aRange[1]) {
					// create array of characters from range
					$aRange = range($aRange[0], $aRange[1]);
				 
					// add to charset array
					$this->aCharSet = array_merge($this->aCharSet, $aRange);
				 }
			  } else {
				 $this->aCharSet[] = $sCurrentItem;
			  }
		   }
		}
	 }
  }
  
  function CaseInsensitive($bCaseInsensitive) {
	 $this->bCaseInsensitive = $bCaseInsensitive;
  }
  
  function SetBackgroundImages($vBackgroundImages) {
	 $this->vBackgroundImages = $vBackgroundImages;
  }
  
  function SetMinFontSize($iMinFontSize) {
	 $this->iMinFontSize = ceil($this->iHeight/2) + 2; //$iMinFontSize;
  }
  
  function SetMaxFontSize($iMaxFontSize) {
	 $this->iMaxFontSize = ceil($this->iHeight/2) + 5;//$iMaxFontSize;
  }
  
  function UseColour($bUseColour) {
	 $this->bUseColour = $bUseColour;
  }
  
  function SetFileType($sFileType) {
	 // check for valid file type
	 if (in_array($sFileType, array('gif', 'png', 'jpeg'))) {
		$this->sFileType = $sFileType;
	 } else {
		$this->sFileType = 'jpeg';
	 }
  }
  
  function DrawLines() {
	 for ($i = 0; $i < $this->iNumLines; $i++) {
		// allocate colour
		if ($this->bUseColour) {
		   $iLineColour = imagecolorallocate($this->oImage, rand(100, 250), rand(100, 250), rand(100, 250));
		} else {
		   $iRandColour = rand(140, 240);
		   $iLineColour = imagecolorallocate($this->oImage, $iRandColour, $iRandColour, $iRandColour);
		}
		
		// draw line
		imageline($this->oImage, rand(0, $this->iWidth), rand($i, $this->iHeight), rand(0, $this->iWidth), rand(0, $this->iHeight), $iLineColour);
		//$p = $i*5;
		//imageline($this->oImage, 0, $p, $this->iWidth, $this->iHeight - $p, $iLineColour);
	 }
	 
	
  }
  
  function DrawOwnerText() {
	 // allocate owner text colour
	 $iBlack = imagecolorallocate($this->oImage, 0, 0, 0);
	 // get height of selected font
	 $iOwnerTextHeight = imagefontheight(2);
	 // calculate overall height
	 $iLineHeight = $this->iHeight - $iOwnerTextHeight - 4;
	 
	 // draw line above text to separate from CAPTCHA
	 imageline($this->oImage, 0, $iLineHeight, $this->iWidth, $iLineHeight, $iBlack);
	 
	 // write owner text
	 imagestring($this->oImage, 2, 3, $this->iHeight - $iOwnerTextHeight - 3, $this->sOwnerText, $iBlack);
	 
	 // reduce available height for drawing CAPTCHA
	 $this->iHeight = $this->iHeight - $iOwnerTextHeight - 5;
  }
  
  function GenerateCode() {
  	 
	 // reset code
	 $this->sCode = '';
	 
	 // loop through and generate the code letter by letter
	 for ($i = 0; $i < $this->iNumChars; $i++) {
		if (count($this->aCharSet) > 0) {
		   // select random character and add to code string
		   $this->sCode .= $this->aCharSet[array_rand($this->aCharSet)];
		} else {
		   // select random character and add to code string
		   $this->sCode .= chr(rand(65, 90));
		}
	 }
	 $this->obj->load->helper('cookie');
	 $this->obj->load->library('encrypt');
	 $code = $this->obj->encrypt->encode($this->sCode);
	 set_cookie($this->sessvar, $code, 0, $this->obj->config->item('cookie_domain'), $this->obj->config->item('cookie_path'));
  }
  
  function DrawCharacters() {
	 // loop through and write out selected number of characters
	 for ($i = 0; $i < strlen($this->sCode); $i++) {
		// select random font
		$sCurrentFont = $this->fontPath.$this->aFonts[array_rand($this->aFonts)];
		
		// select random colour
		if ($this->bUseColour) {
		   $iTextColour = imagecolorallocate($this->oImage, rand(0, 100), rand(0, 100), rand(0, 100));
		
		   if ($this->bCharShadow){
			  // shadow colour
			  $iShadowColour = imagecolorallocate($this->oImage, rand(0, 100), rand(0, 100), rand(0, 100));
		   }
		} else {
		   $iRandColour = 0;
		   $iTextColour = imagecolorallocate($this->oImage, 4, 130, 255);
		
		   if ($this->bCharShadow) {
			  // shadow colour
			  $iRandColour = rand(0, 100);
			  $iShadowColour = imagecolorallocate($this->oImage, $iRandColour, $iRandColour, $iRandColour);
		   }
		}
		
		// select random font size
		$iFontSize = $this->iMinFontSize;
		
		// select random angle
		$iAngle = rand(-10, 10);
		
		// get dimensions of character in selected font and text size
		$aCharDetails = imageftbbox($iFontSize, $iAngle, $sCurrentFont, $this->sCode[$i], array());
		
		// calculate character starting coordinates
		$iX = $this->iSpacing / 8 + $i * $this->iSpacing;
		$iCharHeight = $aCharDetails[2] - $aCharDetails[5];
		$iY = $this->iHeight / 2 + $iCharHeight / 4; 
		
		// write text to image
		imagefttext($this->oImage, $iFontSize, $iAngle, $iX, $iY, $iTextColour, $sCurrentFont, $this->sCode[$i], array());
		
		if ($this->bCharShadow) {
		   $iOffsetAngle = rand(-30, 30);
		   
		   $iRandOffsetX = rand(-5, 5);
		   $iRandOffsetY = rand(-5, 5);
		   
		   imagefttext($this->oImage, $iFontSize, $iOffsetAngle, $iX + $iRandOffsetX, $iY + $iRandOffsetY, $iShadowColour, $sCurrentFont, $this->sCode[$i], array());
		}
	 }
  }
  
  function WriteFile($sFilename) {
	 if ($sFilename == '') {
		// tell browser that data is jpeg
		header("Content-type: image/$this->sFileType");
	 }
	 
	 switch ($this->sFileType) {
		case 'gif':
		   $sFilename != '' ? imagegif($this->oImage, $sFilename) : imagegif($this->oImage);
		   break;
		case 'png':
		   $sFilename != '' ? imagepng($this->oImage, $sFilename) : imagepng($this->oImage);
		   break;
		default:
		   $sFilename != '' ? imagejpeg($this->oImage, $sFilename) : imagejpeg($this->oImage);
	 }
  }
  
  function Create($sFilename = '') {
	 // check for required gd functions
	 if (!function_exists('imagecreate') || !function_exists("image$this->sFileType") || ($this->vBackgroundImages != '' && !function_exists('imagecreatetruecolor'))) {
		return false;
	 }
	 
	 // get background image if specified and copy to CAPTCHA
	 if (is_array($this->vBackgroundImages) || $this->vBackgroundImages != '') {
		// create new image
		$this->oImage = imagecreatetruecolor($this->iWidth, $this->iHeight);
		
		// create background image
		if (is_array($this->vBackgroundImages)) {
		   $iRandImage = array_rand($this->vBackgroundImages);
		   $oBackgroundImage = imagecreatefromjpeg($this->vBackgroundImages[$iRandImage]);
		} else {
		   $oBackgroundImage = imagecreatefromjpeg($this->vBackgroundImages);
		}
		
		// copy background image
		imagecopy($this->oImage, $oBackgroundImage, 0, 0, 0, 0, $this->iWidth, $this->iHeight);
		
		// free memory used to create background image
		imagedestroy($oBackgroundImage);
	 } else {
		// create new image
		$this->oImage = imagecreate($this->iWidth, $this->iHeight);
	 }
	 
	 // allocate white background colour
	 imagecolorallocate($this->oImage, 255, 255, 255);
	 
	 // check for owner text
	 if ($this->sOwnerText != '') {
		$this->DrawOwnerText();
	 }
	 
	 // check for background image before drawing lines
	 if (!is_array($this->vBackgroundImages) && $this->vBackgroundImages == '') {
		$this->DrawLines();
	 }
	 
	 $this->GenerateCode();
	 $this->DrawCharacters();
	 
	 // write out image to file or browser
	 $this->WriteFile($sFilename);
	 
	 // free memory used in creating image
	 imagedestroy($this->oImage);
	 
	 return true;
  }
  
  // call this method statically
  function validate($sUserCode, $bCaseInsensitive = true) {
  	 $this->obj->load->helper('cookie');
	 $this->obj->load->library('encrypt');
	 $data = get_cookie($this->sessvar);
	 $data = $this->obj->encrypt->decode($data);
	
	 
	 if ($bCaseInsensitive) {
		$sUserCode = strtoupper($sUserCode);
		$data = strtoupper($data);
	 }
	 if (!empty($data) && $sUserCode == $data) {
		// clear to prevent re-use
		delete_cookie($this->sessvar);
		
		return true;
	 }
	 
	 return false;
  }
}
   
?>
